package media.gui;

import javax.swing.JFrame;
import javax.swing.JComboBox;
import java.awt.BorderLayout;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import media.config.*;
import media.network.*;

import java.io.File;
import java.lang.Thread;
import java.net.URL;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;


import javax.swing.table.TableCellRenderer;
import javax.swing.table.AbstractTableModel;
import javax.swing.JLabel;
import javax.swing.ImageIcon;
import javax.swing.JTable;
import java.awt.Component;
import java.awt.Cursor;

/**
 * Title:        OcularBase.java<br>
 * Description:  Controls the GUI representation of the program<br>
 * Copyright:    Copyright (c) Konrad Rzeszutek<br>
 * Company:      darnok.dhs.org<br>
 *
 * Version:     $Id: OcularBase.java,v 1.4 2000/11/28 03:41:26 konrad Exp $<br>
 *
 * Controls the ocular representation of the program.<br><br>
 * The usual progress of using this class is following:<p>
 *   OcularBase ocularbase = new OcularBase();<br>
 *   ocularbase.setupProgressBar();<br>
 *   Config config = new Config(ocularbase);<br>
 *   ocularbase.resetProgressBar();<br>
 *   ocularbase.registerConfig(config);<br>
 *   ocularbase.registerNetwork(config);<br>
 *   ocularbase.setupGUI();<br>
 *   ocularbase.setupFinalize();<br>
 *   </p>
 */
public class OcularBase extends JFrame implements WindowListener,
                                                  ActionListener,
                                                  MouseListener{
    /* Mouse cursors */
    private Cursor wait_cursor        = new Cursor(Cursor.WAIT_CURSOR);
    private Cursor default_cursor     = new Cursor(Cursor.DEFAULT_CURSOR);
  /* Progress bar used at startup */
  private JProgressBar bar            = null;

  /* The required buttons */
  private JButton jb_connect          = null;
  private JButton jb_disconnect       = null;
  private JButton jb_setup            = null;
  private JButton jb_quit             = null;


  private JLabel background           = null;
  /* blackboard is used when connecting to a user */
  private JLabel blackboard           = null;
  private JLayeredPane layeredPane     = null;
  private JTextArea status_window     = null;
  private JScrollPane jsp_status      = null;

  /* For setup */
  private JLabel jlb_config_image     = null;

  private JComboBox jcb_schemes       = null;
  private JTextField jtf_scheme_dir   = null;
  private Schemes schemes_obj         = null;

  private JTextField jtf_videofile    = null;
  private JTextField jtf_videoport    = null;
  private JComboBox jcb_video_device  = null;

  private JTextField jtf_audiofile    = null;
  private JTextField jtf_audioport    = null;
  private JComboBox jcb_audio_device  = null;

  private JFileChooser jc              = null;

  private JButton jb_ok_n_save         = null;
  private JButton jb_cancel            = null;

  /* For connection */

  private MyTableModel conn_my_table   = null;
  private TableSorter conn_sorter      = null;
  private JTable conn_table            = null;
  private JScrollPane conn_jsp_table   = null;
  private JButton conn_jb_ok           = null;
  private JButton conn_jb_cancel       = null;

  /* For player */
  private Component visual            = null;
  private Component control           = null;
    private Component my_visual       = null;
  /* All our important information are here */
  private Config config               = null;

  /* Controls Jini, TCP, and RTP */
  private Network network              = null;

  String[] columnNames = {"Status","Name","Hostname"};

  /* state changes - used for the menu buttons */

  public  final int NONE              = 10;
  public  final int SETUP             = 239;
  public  final int CONNECT           = 39;
  public  final int DISCONNECT        = 2831;
  public  final int LOADING           = 281;
  public  final int CONNECTED         = 9321;
  public  int state                   = NONE;

  /* Our preset dimensions */
  private int WIDTH                   = 640;
  private int HEIGHT                  = 429;

  private int GUI_WIDTH               = 630;
  private int GUI_HEIGHT              = 410;

  /* Strictly for JProgressBar */
  private int MAX_TASKS               = 20;
  private int PROGRESS_WIDTH          = 200;
  private int PROGRESS_HEIGHT         = 140;

  /* Counter for progress bar */
  private int current = 0;

  /**
   * Initializes the ocular base of the program.
   * Just setups up the internal variables. Can be done
   * at any time.
   */
  public OcularBase() {

    super("Video Conference");
    this.addWindowListener(this);

    if (layeredPane == null)
      layeredPane = new JLayeredPane();

    super.getContentPane().add(layeredPane);
    super.setSize(new Dimension(WIDTH, HEIGHT));

  }

  /**
   * Registers the Config class.
   * <br><br><b>NOTICE:</b><br><br>
   * Should be done before using:<br><i>
   * <li>setupGUI()
   * <li>setupConfiguration()
   * <li>setupConnect()
   * <li>setupConnectedAudio()
   * <li>setupConnectedVideo()
   * <li>setupPreConnect()
   * </i><br>If not done properly, these methods will
   * throw NullPointerException.
   */
  public void registerConfig(Config c) {

    this.config =c;
    write("Registered configuration subsystem");
  }

    /**
   * Registers the Network class.
   * <br><br><b>NOTICE:</b><br><br>
   * Should be done before using:<br><i>
   * <li>setupConnect()
   * <li>setupConnectedAudio()
   * <li>setupConnectedVideo()
   * </i><br>If not done properly, these methods will
   * throw NullPointerException.
   */
  public void registerNetwork(Network n) {

    this.network = n;
    write("Registered network subsystem");
  }

  /**
   * Setups the progress bar. This should be the first thing done
   * to this class.
   */
  public void setupProgressBar () {

    if (bar == null)
      bar = new JProgressBar(JProgressBar.HORIZONTAL);

    bar.setMaximum(MAX_TASKS);
    bar.setMinimum(0);
    bar.setStringPainted(true);
    /* Some permenant obscure number */
    bar.setBounds(20,20,120,20);

    if (status_window == null)
      status_window = new JTextArea("Starting program ...");

    current = 1;

    status_window.setEditable(false);
    status_window.setBounds(20,60,160,20);

    layeredPane.add(bar, new Integer(1));
    layeredPane.add(status_window, new Integer(1));

    super.setSize(new Dimension(PROGRESS_WIDTH, PROGRESS_HEIGHT));
    super.setVisible(true);
    super.setResizable(false);

    state = LOADING;

  }

  /**
   * Makes the GUI visible.
   */
  public void setupFinalize() {

    super.validate();
    super.setVisible(true);

  }


  /**
   * Setups the configuration toolbox.
   *
   * <br><b>NOTE</b>:<br><br>
   * <i>registerConfig(Config c)</i> <b>MUST</b> be called once
   * prior to using this method or this method will throw
   * a NullPointerException.
   */
   public void setupConfiguration() {


      /* Schemes */
      if (jcb_schemes == null)
        jcb_schemes = new JComboBox();
      else {
        jcb_schemes.removeAllItems();
      }

      String[] str_schemes = config.SCHEMES_OBJ.getSchemeNames();
      int choosen = 0;
      for (int loop0 = 0; loop0 < str_schemes.length; loop0++) {
          jcb_schemes.addItem(str_schemes[loop0]);
          if (str_schemes[loop0].compareTo(config.SCHEME) == 0)
            choosen = loop0;
      }
      jcb_schemes.setSelectedIndex(choosen);
      jcb_schemes.setBounds(config.SCHEME_OBJ.player_boundary.x+180,
                            config.SCHEME_OBJ.player_boundary.y+10,
                            160,20);
      jcb_schemes.setOpaque(false);

      /* Scheme directory */
      if (jtf_scheme_dir == null) {
        jtf_scheme_dir = new JTextField(config.SCHEME_DIR);
        jtf_scheme_dir.addMouseListener(this);
      }
      else
        jtf_scheme_dir.setText(config.SCHEME_DIR);

      jtf_scheme_dir.setBounds(config.SCHEME_OBJ.player_boundary.x+180,
                               config.SCHEME_OBJ.player_boundary.y+44,
                               160,20);
      jtf_scheme_dir.setForeground(Color.white);
      jtf_scheme_dir.setBackground(Color.black);
      jtf_scheme_dir.setBorder(null);


      /* VideoInput File */
      if (jtf_videofile == null) {
        jtf_videofile = new JTextField(config.VIDEO_INPUT);
        jtf_videofile.addMouseListener(this);
      }
      else
        jtf_videofile.setText(config.VIDEO_INPUT);

      jtf_videofile.setBounds(config.SCHEME_OBJ.player_boundary.x+180,
                              config.SCHEME_OBJ.player_boundary.y+127,
                              160,20);
      jtf_videofile.setForeground(Color.white);
      jtf_videofile.setBackground(Color.black);
      jtf_videofile.setBorder(null);


      /* Audio Input file */
      if (jtf_audiofile == null) {
        jtf_audiofile = new JTextField(config.AUDIO_INPUT);
        jtf_audiofile.addMouseListener(this);
      }
      else
        jtf_audiofile.setText(config.AUDIO_INPUT);
      jtf_audiofile.setBounds(config.SCHEME_OBJ.player_boundary.x+180,
                              config.SCHEME_OBJ.player_boundary.y+225,
                              160,20);
      jtf_audiofile.setForeground(Color.white);
      jtf_audiofile.setBackground(Color.black);
      jtf_audiofile.setBorder(null);


      /* VideoPort */
      if (jtf_videoport == null) {
        jtf_videoport = new JTextField(String.valueOf(config.VIDEO_PORT));
        jtf_videoport.addActionListener(this);
      }
      else
        jtf_videoport.setText(String.valueOf(config.VIDEO_PORT));
      jtf_videoport.setBounds(config.SCHEME_OBJ.player_boundary.x+180,
                              config.SCHEME_OBJ.player_boundary.y+73,
                              160,20);
      jtf_videoport.setForeground(Color.white);
      jtf_videoport.setBackground(Color.black);
      jtf_videoport.setBorder(null);

      /* AudioPort */
      if (jtf_audioport == null) {
        jtf_audioport = new JTextField(String.valueOf(config.AUDIO_PORT));
        jtf_audioport.addActionListener(this);
      }
      else
        jtf_audioport.setText(String.valueOf(config.AUDIO_PORT));

      jtf_audioport.setBounds(config.SCHEME_OBJ.player_boundary.x+180,
                              config.SCHEME_OBJ.player_boundary.y+172,
                              160,20);
      jtf_audioport.setForeground(Color.white);
      jtf_audioport.setBackground(Color.black);
      jtf_audioport.setBorder(null);

      /* VideoDevice */
      if (jcb_video_device == null) {
        jcb_video_device = new JComboBox(config.VIDEO);
	// Set the right device
	for (int loop0 =0;loop0< config.VIDEO.length;loop0++) 
	    if (config.VIDEO_DRIVER.equalsIgnoreCase(config.VIDEO[loop0]))
		jcb_video_device.setSelectedIndex(loop0);
        jcb_video_device.addActionListener(this);
      }
      else {
        jcb_video_device.removeAllItems();
        for (int loop0 = 0; loop0 < config.VIDEO.length; loop0++) {
          jcb_video_device.addItem(config.VIDEO[loop0]);
	  if (config.VIDEO_DRIVER.equalsIgnoreCase(config.VIDEO[loop0]))
	      jcb_video_device.setSelectedIndex(loop0);
	}
	// Set the current one.

      }

      jcb_video_device.setBounds(config.SCHEME_OBJ.player_boundary.x+180,
                                 config.SCHEME_OBJ.player_boundary.y+94,
                                 160,20);
      jcb_video_device.setOpaque(false);


      /* AudioDevice */
      if (jcb_audio_device == null) {
        jcb_audio_device = new JComboBox(config.AUDIO);
	 for (int loop0 = 0; loop0 < config.AUDIO.length; loop0++) 
	  if (config.AUDIO_DRIVER.equalsIgnoreCase(config.AUDIO[loop0]))
	      jcb_audio_device.setSelectedIndex(loop0);
        jcb_audio_device.addActionListener(this);
      }
      else {
        jcb_audio_device.removeAllItems();
        for (int loop0 = 0; loop0 < config.AUDIO.length; loop0++) {
          jcb_audio_device.addItem(config.AUDIO[loop0]);
	  if (config.AUDIO_DRIVER.equalsIgnoreCase(config.AUDIO[loop0]))
	      jcb_audio_device.setSelectedIndex(loop0);
	}
      }
      jcb_audio_device.setBounds(config.SCHEME_OBJ.player_boundary.x+180,
                                 config.SCHEME_OBJ.player_boundary.y+195,
                                 160,20);
      jcb_audio_device.setOpaque(false);


      /* Background picture */
      if (jlb_config_image == null)
        jlb_config_image = new JLabel(config.SCHEME_OBJ.configure_icon);
      else
        jlb_config_image.setIcon(config.SCHEME_OBJ.configure_icon);

      jlb_config_image.setOpaque(false);
      jlb_config_image.setBorder(null);
      jlb_config_image.setBounds(config.SCHEME_OBJ.player_boundary.x,
                                 config.SCHEME_OBJ.player_boundary.y,
                                 352,288);

      /* Ok and Save */
      if (jb_ok_n_save == null) {
      // CHECK make this icons?
        jb_ok_n_save = new JButton("Ok and Save");
        jb_ok_n_save.setToolTipText("... save these settings to be default ...");
        jb_ok_n_save.addActionListener(this);
      }

      jb_ok_n_save.setBorder(null);
      jb_ok_n_save.setOpaque(false);
      jb_ok_n_save.setForeground(Color.white);
      jb_ok_n_save.setBackground(Color.black);
      jb_ok_n_save.setBounds(config.SCHEME_OBJ.player_boundary.x+40,
                             config.SCHEME_OBJ.player_boundary.y+255,
                             120,20);


      if (jb_cancel == null) {
      // CHECK
        jb_cancel = new JButton("Cancel");
        jb_cancel.setToolTipText("... wanna keep the way it is now ...");
        jb_cancel.addActionListener(this);
      }

      jb_cancel.setBorder(null);
      jb_cancel.setOpaque(false);
      jb_cancel.setBounds(config.SCHEME_OBJ.player_boundary.x+220,
                          config.SCHEME_OBJ.player_boundary.y+255,
                          60,20);
      jb_cancel.setForeground(Color.white);
      jb_cancel.setBackground(Color.black);


      layeredPane.add(jlb_config_image, new Integer(1));
      layeredPane.add(jcb_schemes, new Integer(2));
      layeredPane.add(jtf_scheme_dir, new Integer(2));

      layeredPane.add(jtf_videoport, new Integer(2));
      layeredPane.add(jcb_video_device, new Integer(2));
      layeredPane.add(jtf_videofile, new Integer(2));

      layeredPane.add(jtf_audioport, new Integer(2));
      layeredPane.add(jcb_audio_device, new Integer(2));
      layeredPane.add(jtf_audiofile, new Integer(2));

      layeredPane.add(jb_ok_n_save, new Integer(2));
      layeredPane.add(jb_cancel, new Integer(2));

      state = SETUP;

      repaint();
   }


  /**
   * Setups the GUI.
   * <br><b>NOTE</b>:<br><br>
   * <i>registerConfig(Config c)</i> <b>MUST</b> be called once
   * prior to using this method or this method will throw
   * a NullPointerException.
   */

  public void setupGUI() {

      state = NONE;
      super.setSize(new Dimension(WIDTH, HEIGHT));
      super.setResizable(false);


      if (jb_connect == null) {
        jb_connect = new JButton(config.SCHEME_OBJ.connect_icon);
        jb_connect.addActionListener(this);
      }
      else
        jb_connect.setIcon(config.SCHEME_OBJ.connect_icon);

      jb_connect.setBorderPainted(false);
      jb_connect.setOpaque(false);
      jb_connect.setBorder(null);
      jb_connect.setBackground(Color.black);
      jb_connect.setForeground(Color.black);
      jb_connect.setToolTipText("... connect to somebody ...");
      jb_connect.setBounds(config.SCHEME_OBJ.connect_boundary);

      if (jb_disconnect == null){
        jb_disconnect = new JButton(config.SCHEME_OBJ.disconnect_icon);
        jb_disconnect.addActionListener(this);
      }
      else
        jb_disconnect.setIcon(config.SCHEME_OBJ.disconnect_icon);

      jb_disconnect.setBorderPainted(false);
      jb_disconnect.setOpaque(false);
      jb_disconnect.setBorder(null);
      jb_disconnect.setBackground(Color.black);
      jb_disconnect.setForeground(Color.black);
      jb_disconnect.setBounds(config.SCHEME_OBJ.disconnect_boundary);
      jb_disconnect.setToolTipText("... disconnect ...");

      if (jb_setup == null) {
        jb_setup = new JButton(config.SCHEME_OBJ.setup_icon);
        jb_setup.addActionListener(this);
      }
      else
        jb_setup.setIcon(config.SCHEME_OBJ.setup_icon);

      jb_setup.setBorderPainted(false);
      jb_setup.setOpaque(false);
      jb_setup.setBorder(null);
      jb_setup.setBackground(Color.black);
      jb_setup.setForeground(Color.black);
      jb_setup.setBounds(config.SCHEME_OBJ.setup_boundary);
      jb_setup.setToolTipText("... reconfigure this program ...");

      if (jb_quit == null) {
        jb_quit = new JButton(config.SCHEME_OBJ.quit_icon);
        jb_quit.addActionListener(this);
      }
      else
        jb_quit.setIcon(config.SCHEME_OBJ.quit_icon);

      jb_quit.setBorderPainted(false);
      jb_quit.setOpaque(false);
      jb_quit.setFocusPainted(false);
      jb_quit.setBorder(null);
      jb_quit.setBackground(Color.black);
      jb_quit.setForeground(Color.black);
      jb_quit.setBounds(config.SCHEME_OBJ.quit_boundary);
      jb_quit.setToolTipText("... go away ...");

      if (background == null)
        background = new JLabel(config.SCHEME_OBJ.background_icon);
      else
        background.setIcon(config.SCHEME_OBJ.background_icon);

      background.setBounds(0, 0, config.SCHEME_OBJ.background_icon.getIconWidth(),
                                 config.SCHEME_OBJ.background_icon.getIconHeight());

      if (status_window == null)
        status_window = new JTextArea("Loaded scheme: "+config.SCHEME+"\n",5,20);
      else
        write("Loaded scheme: "+config.SCHEME);

      status_window.setForeground(Color.white);
      status_window.setBackground(Color.black);
      status_window.setWrapStyleWord(true);
      status_window.setLineWrap(true);
      status_window.setAlignmentY(status_window.BOTTOM_ALIGNMENT);

      if (jsp_status == null)
        jsp_status= new JScrollPane(status_window);

      jsp_status.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

      jsp_status.setBounds(config.SCHEME_OBJ.status_boundary.x,
                           config.SCHEME_OBJ.status_boundary.y,170,60);
      jsp_status.setBackground(Color.white);
      jsp_status.setBackground(Color.black);
      jsp_status.setAlignmentY(JScrollPane.BOTTOM_ALIGNMENT);


      if (layeredPane == null)
        layeredPane = new JLayeredPane();
      else
        layeredPane.removeAll();

      layeredPane.setPreferredSize(new Dimension(GUI_WIDTH,GUI_HEIGHT));
      layeredPane.setForeground(Color.black);
      layeredPane.setBackground(Color.black);

      layeredPane.add(background, new Integer(0));
      layeredPane.add(jb_connect, new Integer(1));
      layeredPane.add(jb_disconnect, new Integer(1));
      layeredPane.add(jb_setup, new Integer(1));
      layeredPane.add(jb_quit, new Integer(1));
      layeredPane.add(jsp_status, new Integer(1));

      repaint();
  }



  /**
   * This internal class is used for JTable. This class
   * allows JTable to interpret ImageIcon fields as ImageIcon
   * as opposed to some string.
   */
  class MyTableModel extends AbstractTableModel {

        Object[][] data;

        public MyTableModel(Network network) {
          // This should automaticly get data.

          this.data = network.getUsers();

        }

        /**
         * Refresh teh list of users.
         */
        public void refresh() {

          this.data = network.getUsers();
        }

        public int getColumnCount() {
            return columnNames.length;
        }

        public int getRowCount() {
            return data.length;
        }

        public String getColumnName(int col) {
            return columnNames[col];
        }

        public Object getValueAt(int row, int col) {
            return data[row][col];
        }

        /*
         * JTable uses this method to determine the default renderer/
         * editor for each cell.  If we didn't implement this method,
         * then the ImageIcon would be: "javax.swing.ImageIcon@232324"
         * rather than a picture.
         */
        public Class getColumnClass(int c) {
            return getValueAt(0, c).getClass();
        }
   }

   /**
    * Class used to render ImageIcons.
    */
   class ImageIconRenderer extends JLabel
                        implements TableCellRenderer {

        public ImageIconRenderer(boolean isBordered) {
            super();
            setOpaque(true);
        }
        /**
         * Sets the attributes of the JLabel depending on the given
         * attributes.
         */
        public Component getTableCellRendererComponent(
                                JTable table, Object icon,
                                boolean isSelected, boolean hasFocus,
                                int row, int column) {
          this.setBackground(Color.black);
          this.setForeground(Color.black);
          this.setIcon((ImageIcon)icon);
          return this;
        }
    }

  /**
   * Setups the GUI for the connection - uses JTables.
   * <br><b>NOTE</b>:<br><br>
   * <i>registerConfig(Config c)</i> <b>MUST</b> be called once
   * prior to using this method or this method will throw
   * a NullPointerException.
   */
  public void setupConnect() {


      if (conn_my_table == null)
        conn_my_table = new MyTableModel(network);
      else
        conn_my_table.refresh();

      if (conn_sorter == null)
        conn_sorter = new TableSorter(conn_my_table);
      else
	  conn_sorter.setModel(conn_my_table);
      if (conn_table == null)
        conn_table = new JTable(conn_sorter);


      conn_sorter.addMouseListenerToHeaderInTable(conn_table);

      conn_table.setDefaultRenderer(ImageIcon.class,
                                    new ImageIconRenderer(true));

      conn_table.getColumnModel().getColumn(0).setPreferredWidth(40);
      conn_table.getColumnModel().getColumn(1).setPreferredWidth(80);
      conn_table.setForeground(Color.white);
      conn_table.setOpaque(false);
      conn_table.setBackground(Color.black);
      conn_table.setShowGrid(false);
      if (conn_jsp_table == null)
        conn_jsp_table = new JScrollPane(conn_table);

      conn_jsp_table.setVerticalScrollBarPolicy(
		JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

      conn_jsp_table.setBounds(config.SCHEME_OBJ.player_boundary.x,
                               config.SCHEME_OBJ.player_boundary.y,
                               352,288);
      conn_jsp_table.setBackground(Color.black);
      conn_jsp_table.setForeground(Color.white);

      /* Connect button */
      if (conn_jb_ok == null) {
        conn_jb_ok = new JButton(config.SCHEME_OBJ.connect_ok_icon);
        conn_jb_ok.addActionListener(this);
      } else
        conn_jb_ok.setIcon(config.SCHEME_OBJ.connect_ok_icon);

      conn_jb_ok.setBorder(null);
      conn_jb_ok.setOpaque(false);
      conn_jb_ok.setForeground(Color.white);
      conn_jb_ok.setBackground(Color.black);
      conn_jb_ok.setBounds(config.SCHEME_OBJ.connect_ok_boundary);

      /* Cancel button */
      if (conn_jb_cancel == null) {
        conn_jb_cancel = new JButton(config.SCHEME_OBJ.connect_cancel_icon);
        conn_jb_cancel.addActionListener(this);
      } else
        conn_jb_cancel.setIcon(config.SCHEME_OBJ.connect_cancel_icon);

      conn_jb_cancel.setBorder(null);
      conn_jb_cancel.setOpaque(false);
      conn_jb_cancel.setBounds(config.SCHEME_OBJ.connect_cancel_boundary);
      conn_jb_cancel.setForeground(Color.white);
      conn_jb_cancel.setBackground(Color.black);

      layeredPane.add(conn_jsp_table, new Integer(2));
      layeredPane.add(conn_jb_ok, new Integer(2));
      layeredPane.add(conn_jb_cancel, new Integer(2));

      repaint();
  }

  /**
   * Setups a pre-connect billboard screen.
   * <br><b>NOTE</b>:<br><br>
   * <i>registerConfig(Config c)</i> and
   * <i>registerNetwork(Network n)</i>
   * <b>MUST</b> be called once prior to using this
   * method or this method will throw a NullPointerException.
   */
  public void setupPreConnect() {

      if (blackboard == null)
        blackboard = new JLabel("         Trying to connect .... ");

      blackboard.setBackground(Color.black);
      blackboard.setForeground(Color.white);
      blackboard.setOpaque(true);
      blackboard.setBorder(null);

      blackboard.setBounds(config.SCHEME_OBJ.player_boundary.x,
                           config.SCHEME_OBJ.player_boundary.y,
                           352,288);

      bar.setMaximum(10);
      bar.setMinimum(0);
      bar.setStringPainted(true);
    /* Some permenant obscure number */
      bar.setBounds(config.SCHEME_OBJ.player_boundary.x+20,
		    config.SCHEME_OBJ.player_boundary.y+20,
		    120,20);
      current = 1;
      bar.setValue(current);

      layeredPane.add(blackboard, new Integer(1));
      layeredPane.add(bar, new Integer(2));

      repaint();
  }

  /**
   * Setups a video screen for the incoming RTP session.
   * <br><b>NOTE</b>:<br><br>
   * <i>registerConfig(Config c)</i> and
   * <i>registerNetwork(Network n)</i>
   * <b>MUST</b> be called once prior to using this
   * method or this method will throw a NullPointerException.
   */
  public void setupConnectedVideo(Component visual) {


      this.visual = visual;

      visual.setBounds(config.SCHEME_OBJ.player_boundary.x,
                      config.SCHEME_OBJ.player_boundary.y,
                      352,288);

      layeredPane.add(visual, new Integer(2));

  }

    public void setupMyVideo(Component visual) {
	
	this.my_visual = visual;
	my_visual.setBounds(config.SCHEME_OBJ.player_boundary.x,
			    config.SCHEME_OBJ.player_boundary.y,
			    160,140);
	layeredPane.add(my_visual, new Integer(2));
    }
	
  /**
   * Setups a audio control compnenet for the incoming audio session.
   * <br><b>NOTE</b>:<br><br>
   * <i>registerConfig(Config c)</i> and
   * <i>registerNetwork(Network n)</i>
   * <b>MUST</b> be called once prior to using this
   * method or this method will throw a NullPointerException.
   */
  public void setupConnectedAudio(Component control) {

      this.control = control;
      control.setBounds(config.SCHEME_OBJ.player_boundary.x,
                        config.SCHEME_OBJ.player_boundary.y+288,
                        120,20);

      layeredPane.add(control, new Integer(2));

  }

  /**
   * Removes the progress bar from the screen.
   */
   public void resetProgressBar() {

    this.setVisible(false);
    current = 0;
    layeredPane.removeAll();


  }

  /**
   * Resets the GUI.
   */
  public void resetGUI() {

    this.setVisible(false);
    layeredPane.removeAll();

  }

  /**
   * Clears the GUI so that the configuration (setup) screen disappears.
   */
  public void resetConfiguration() {

      layeredPane.remove(jlb_config_image);
      layeredPane.remove(jcb_schemes);
      layeredPane.remove(jtf_scheme_dir);

      layeredPane.remove(jtf_videoport);
      layeredPane.remove(jcb_video_device);
      layeredPane.remove(jtf_videofile);

      layeredPane.remove(jtf_audioport);
      layeredPane.remove(jcb_audio_device);
      layeredPane.remove(jtf_audiofile);
      layeredPane.remove(jb_ok_n_save);
      layeredPane.remove(jb_cancel);

      layeredPane.remove(background);
      layeredPane.add(background, new Integer(0));

      repaint();
  }

  /**
   * clears the GUI so that the connection screen disapperars.
   */
  public void resetConnect () {


      layeredPane.remove(conn_jb_ok);
      layeredPane.remove(conn_jb_cancel);
      layeredPane.remove(conn_jsp_table);

      layeredPane.setPreferredSize(new Dimension(610,410));
      layeredPane.setForeground(Color.black);
      layeredPane.setBackground(Color.black);

      repaint();
  }

  /**
   * Reset connected Video and Audio components.
   */

  public void resetConnected() {

      if (blackboard != null) {
        layeredPane.remove(blackboard);
	layeredPane.remove(bar);
      }
      if (visual != null) {

        layeredPane.remove(visual);
      }
      if (my_visual != null ) 
	  layeredPane.remove(my_visual);
      if (control != null)
        layeredPane.remove(control);

      my_visual = null;
      visual = null;
      control = null;


      repaint();
  }


  /**
   * Writes out diagnostic and usefull information to the
   * status window.
   */
  public void write(String s) {

    //if (status_window == null)
      System.out.println(" ["+s+"]");
   // else {

      if (state == LOADING) {
        current++;
        status_window.setText(s+" ...");
        bar.setValue(current);
      }
      else {
	  if (state != CONNECTED)
	      status_window.append(s+"\n");
	  else {
	      current++;
	      bar.setValue(current);
	  }
      }
  }

  /**
   * Provides a graphical mean to inform the user about errors.
   * <br>When called, the string <i>s</i> is pasted into JOptionPane
   * and the user has two choices: Continue or Cancel. If the user
   * chooses the latter, true is returned. Otherwise true.
   * <br><br><b>return</b>:<br>
   *  true iff user selects 'Cancel'<br>
   *  false otherwise
   */
  public boolean error(String s) {


      Object[] options = {"Continue","Cancel"};

      System.out.println("[" +s+"]");

      int n = JOptionPane.showOptionDialog(this,
                                   s,
                                   "Bummer!",
                                   JOptionPane.YES_NO_CANCEL_OPTION,
                                   JOptionPane.ERROR_MESSAGE,
                                   null,
                                   options,
                                   options[1]);
      return (true ? n == 1 : n == 0);
  }

    public void info(String s) {

	JOptionPane.showMessageDialog(this, s);


    }

 public boolean infoDecision(String s) {


      Object[] options = {"Speak with him","Tell him to buzz off"};

      int n = JOptionPane.showOptionDialog(this,
                                   s,
                                   "Decisons ",
                                   JOptionPane.YES_NO_CANCEL_OPTION,
					   JOptionPane.QUESTION_MESSAGE,
                                   null,
                                   options,
                                   options[1]);
      return (true ? n == 1 : n == 0);
  }
   /**
    * Awaits user inputs.
    */
   public void actionPerformed(ActionEvent e){

    Object obj = e.getSource();

    if (obj instanceof JButton) {

      /* Setup buttons */
      if (obj == jb_ok_n_save) {

        write("Changing scheme");
        config.SCHEME = (String)jcb_schemes.getSelectedItem();
        config.SCHEME_DIR = jtf_scheme_dir.getText();

        config.VIDEO_INPUT = jtf_videofile.getText();
        config.VIDEO_PORT = Integer.valueOf(jtf_videoport.getText()).intValue();
        config.VIDEO_DRIVER = (String)jcb_video_device.getSelectedItem();

        config.AUDIO_INPUT = jtf_audiofile.getText();
        config.AUDIO_PORT = Integer.valueOf(jtf_audioport.getText()).intValue();
        config.AUDIO_DRIVER = (String)jcb_audio_device.getSelectedItem();

        try {
          config.writeConfigFile();
          config.readConfigFile();

          resetGUI();
          setupGUI();
          setupConfiguration();
          super.getContentPane().add(layeredPane); this.setVisible(true);

        } catch (Exception error) {
           JOptionPane.showMessageDialog(null,
                                          //JOptionPane.ERROR_MESSAGE,
                                          error.getMessage());
        }
      }
      /* This is cancel button from the Setup! */
      if (obj == jb_cancel) {
        resetConfiguration();
        state = NONE;
      }

      /* The other three buttons from the main menu */

      if (obj == jb_quit)
        System.exit(0);

      if (obj == jb_setup) {

        if (state == CONNECT) {
          resetConnect();
          state = SETUP;
          setupConfiguration();
        }
        if (state == SETUP) {
          resetConfiguration();
          state = NONE;
        }
        if (state == NONE) {
          setupConfiguration();
          state = SETUP;
        }
	if (state == CONNECTED) {
	    // Won't do anything,
	}
      }
      /* The connect button from the main menu */
      if (obj == jb_connect) {

        if (state == CONNECT) {
          resetConnect();
          state = NONE;
        }
        if (state == SETUP) {
          resetConfiguration();
          state = CONNECT;
          setupConnect();
        }
        if (state == NONE) {
          setupConnect();
          state = CONNECT;
        }
	if (state == CONNECTED) {
	    setupConnect();
	    //state = CONNECT;
	}
      }

      /* This is performed when the user wants to speak
         with some dude */
      if (obj == conn_jb_ok) {
                /* Unfortunatly the user migth move the colums around,
                thus we must find the right column. */
                int row = conn_table.getSelectedRow();
                if (row != -1) {

                  String hostname = null;
                  String username = null;

                  for (int j=0; j< 3; j++)
                    if (conn_table.getValueAt(row, j) instanceof String) {
                      String boo = (String)conn_table.getValueAt(row, j);
                      if (boo.indexOf('@') != -1)
                        username = boo;
                      else
                        hostname = boo;
		    }
                  write("Connecting with "+username+" ["+hostname+"]");
		  //network.disconnect();
                  resetConnect();
                  setupPreConnect();
		  super.getContentPane().setCursor(wait_cursor);

		  if (network.connect(hostname, username) == network.OK) {
		      state = CONNECTED;
		      config.write("OK!");
		  }
		  else {
		      info("Couldn't connect to "+hostname);
		      config.write("ERROR");
		      network.disconnect();
		      resetConnected();
		      state = NONE;
		  }
		  getContentPane().setCursor(default_cursor);

		}
                  else {

                  write("Defaulting to multicast connection");
                  resetConnect();
                  setupPreConnect();
                  network.connect("","");
		  state = CONNECTED;
                }
      }
      if (obj == jb_disconnect) {
        if (state == CONNECTED) {	  
	    if (network.disconnect() != network.OK) 
		write("Had problems closing the AV RTP");
          resetConnected();
	  write("Closed connection");
	  state = NONE;
        }
	if (state == CONNECT) {
	    resetConnect();
	    state = NONE;
	}
      }

      /* This 'Cancel' button is only up when the user doesn't
         want to select a connection (in the connection menu)*/
      if (obj == conn_jb_cancel) {
      	if (state == CONNECT) {
      	    resetConnect();
      	    state = NONE;
      	 }
      }
    }
   }

  /**
   * Perform specific operations when user clicks on some of the
   * JTextFields. Only used by the setup screen.
   */
   public void mouseClicked(MouseEvent e) {

    Object obj = e.getSource();
    int result;

    if (jc == null)
      jc = new JFileChooser();

    if (obj instanceof JTextField) {
      if (obj == jtf_videofile) {
        jc.setDialogTitle("Pick a video file");
        jc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        result = jc.showOpenDialog(this);
        if (result == JFileChooser.APPROVE_OPTION)

          jtf_videofile.setText(jc.getSelectedFile().getAbsolutePath().replace('\\','/'));
        }

      if (obj == jtf_audiofile) {
        jc.setDialogTitle("Pick a audio file");
        jc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
        result = jc.showOpenDialog(this);
        if (result == JFileChooser.APPROVE_OPTION)
          jtf_audiofile.setText(jc.getSelectedFile().getAbsolutePath().replace('\\','/'));
      }
      if (obj == jtf_scheme_dir) {
        jc.setCurrentDirectory(new File(config.SCHEME_DIR));
        jc.setDialogTitle("Choose a directory where schemes reside");
        jc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
        result = jc.showOpenDialog(this);
        if (result == JFileChooser.APPROVE_OPTION) {
          try {
            schemes_obj = new Schemes(jc.getSelectedFile().getAbsolutePath().replace('\\','/'), null);
            jtf_scheme_dir.setText(jc.getSelectedFile().getAbsolutePath().replace('\\','/'));
         } catch (Exception error) {
            JOptionPane.showMessageDialog(null,
                                          //JOptionPane.ERROR_MESSAGE,
                                          error.getMessage());
          }
         }
      }
     }
   }

  public void mousePressed(MouseEvent e) {  }
  public void mouseReleased(MouseEvent e){  }
  public void mouseEntered(MouseEvent e) {  }
  public void mouseExited(MouseEvent e)  {  }


  public void windowOpened(WindowEvent e) {}
  public void windowClosing(WindowEvent e) {      System.exit(0);}
  public void windowClosed(WindowEvent e) {
      this.dispose();
      System.exit(0);
  }
  public void windowIconified(WindowEvent e) {}
  public void windowDeiconified(WindowEvent e) {}
  public void windowActivated(WindowEvent e) {
      }
  public void windowDeactivated(WindowEvent e) { }


}
